@@ -172,9 +172,9 @@ eDT <- function(
 #' @importFrom shiny moduleServer observe reactiveValues reactive 
 #'  observeEvent actionButton icon renderPrint showNotification req
 #'  isolate is.reactive modalDialog modalButton renderUI uiOutput showModal
-#'  freezeReactiveValue
+#'  freezeReactiveValue isTruthy
 #' @importFrom DT dataTableProxy renderDT formatStyle styleEqual hideCols
-#' @importFrom dplyr collect %>% relocate rows_update rows_insert rows_delete is.tbl all_of tibble
+#' @importFrom dplyr collect %>% relocate is.tbl all_of tibble
 #' @importFrom utils str tail
 #' @importFrom uuid UUIDgenerate
 #' @importFrom shinyjs disable enable
@@ -407,9 +407,8 @@ eDTServer <- function(
         # rv$newState gets assigned by various actions in the app.
         observe(label = "Replace front-end data",{
               rv$triggerNewState
-              req(rv$newState)
+              req(!is.null(rv$newState) && isTruthy(rv$newState))
               castCols <- base::colnames(isolate(data()))
-              
               data <- rv$newState
               data <- relocate(data,  dplyr::all_of("buttons"))     
               rv$modifiedData <- data
@@ -617,7 +616,7 @@ eDTServer <- function(
             })
         
         observe({
-              req(rv$changelog)
+              req(!is.null(rv$changelog) && isTruthy(rv$changelog))
               rv$changelog_react
               
               rv$changeLogTracker <- length(rv$changelog)
@@ -705,7 +704,7 @@ eDTServer <- function(
             })
         
         observeEvent(input$DT_cells_filled, {      
-              req(input$DT_cells_filled)
+              req(!is.null(input$DT_cells_filled) && isTruthy(input$DT_cells_filled))
               edits <- input$DT_cells_filled
               edits$row <- edits$row + min(input$DT_rows_current -1)
               rv$edits <- edits
@@ -718,7 +717,7 @@ eDTServer <- function(
             })
          
         observeEvent(rv$edits_react, {
-              req(rv$edits)
+              req(!is.null(rv$edits) && isTruthy(rv$edits))
               edits <- unique(rv$edits)
               rv$edits <- NULL
               
@@ -908,7 +907,7 @@ eDTServer <- function(
             })
         
         observeEvent(input$confirmCommit, {
-              req(effectiveChanges())
+              req(!is.null(effectiveChanges()) && isTruthy(effectiveChanges()))
               modified <- effectiveChanges()
               cols <- as.character(dplyr::tbl_vars(data()))
               checkPoint <- rv$checkPointData
@@ -949,25 +948,65 @@ eDTServer <- function(
                     inserted <- inserted[,cols]
                     
                     if(nrow(deleted)){
+                      if(inherits(result, 'tbl_dbi')){
+                        # dbplyr:::rows_delete.tbl_dbi requires y to be in the same source and will start a transaction for this if not the case.
+                        # Most backends do not support nested transactions. Therefore manually copy the data first.
+                        temp_name = paste0("editbl_", gsub("-", "", UUIDgenerate()))
+                        deleted <- dplyr::copy_to(
+                            dest = result$src,
+                            df = deleted,
+                            name = temp_name,
+                            temporary = TRUE,
+                            in_transaction = FALSE)
+                      }
+                      
                       result <- rows_delete(
                           x = result,
                           y = deleted,
                           by = keys(),
+                          in_place = in_place(),
+                          unmatched = 'ignore')
+                      
+                      if(inherits(result, 'tbl_dbi')){
+                        DBI::dbRemoveTable(result$src$con, temp_name)
+                      }
+                    }
+                    if(nrow(edited)){
+                      result <- e_rows_update(
+                          x = result,
+                          y = edited,
+                          match = match,
+                          by = keys(),
                           in_place = in_place())
                     }
                     
-                    result <- rows_update(
-                        x = result,
-                        y = edited,
-                        match = match,
-                        by = keys(),
-                        in_place = in_place())
-                    
-                    result <- rows_insert(
-                        x = result,
-                        y = inserted,
-                        by = keys(),
-                        in_place = in_place())
+                    if(nrow(inserted)){
+# Needed code should there be a switch to dbplyr::rows_insert
+#                      if(inherits(result, 'tbl_dbi')){
+#                        # https://github.com/openanalytics/editbl/issues/1
+#                        # dbplyr:::rows_insert.tbl_dbi requires y to be in the same source and will start a transaction for this if not the case.
+#                        # Most backends do not support nested transactions. Therefore manually copy the data first.
+#                        temp_name = paste0("editbl_", gsub("-", "", UUIDgenerate()))
+#                        inserted <- dplyr::copy_to(
+#                            dest = result$src,
+#                            df = inserted,
+#                            name = temp_name,
+#                            temporary = TRUE,
+#                            in_transaction = FALSE)
+#                      }
+                      
+                      result <- e_rows_insert(
+                          x = result,
+                          y = inserted,
+                          by = keys(),
+                          in_place = in_place(),
+                          conflict = "ignore")
+                      
+# Needed code should there be a switch to dbplyr::rows_insert                     
+#                      if(inherits(result, 'tbl_dbi')){
+#                        DBI::dbRemoveTable(result$src$con, temp_name)
+#                      }       
+                    }
                     
                     rv$committedData <- result
                     
@@ -991,11 +1030,11 @@ eDTServer <- function(
                     
                     commitTransaction(result)
                     rv$resultMessage <- "success."
-                  }, error = function(cond){
+                  }, error = function(cond){                    
                     rollbackTransaction(result)
                     rv$resultMessage <- sprintf(
                         "Failure: %s",
-                        cond$message
+                        conditionMessage(cond)
                     )
                   })
               rv$showResultMessage <- Sys.time()
@@ -1021,8 +1060,8 @@ eDTServer <- function(
         
         # Ensure selection holds while deleting / adding rows
         observe(priority = 1,{
-              req(rv$modifiedData)
-              req(isolate(rv$selected))
+              req(!is.null(rv$modifiedData) && isTruthy(rv$modifiedData))
+              req(!is.null(isolate(rv$selected)) && isTruthy(isolate(rv$selected)))
 
               currentSelection <- isolate(rv$selected)$i  
               newIndexes <- which(rv$modifiedData$i %in% currentSelection)
